
GLOBALS_SECTION
  #include <time.h>
  time_t start,finish;
  long hour,minute,second;
  double elapsed_time;
  char s1[2] = {" "};

TOP_OF_MAIN_SECTION
  arrmblsize=1000000;
  gradient_structure::set_MAX_NVAR_OFFSET(6000);
  gradient_structure::set_GRADSTACK_BUFFER_SIZE(10000000);
  gradient_structure::set_CMPDIF_BUFFER_SIZE(10000000);
  time(&start);

DATA_SECTION
  init_int nobs;
  init_matrix data(1,nobs,1,3)
  init_number m
  init_int pryears
  init_number prcatch

  vector obs_catch(1,nobs);
  vector cpue(1,nobs);
  vector effort(1,nobs);
  vector years(1,nobs);
  number avg_effort
  number avg_effort_p
  //!!rec_seed = 1211;

INITIALIZATION_SECTION
  q 0.02
  beta 0.8
  r 0.1
  log_binit 15
  cv_C 0.25

PARAMETER_SECTION
  init_bounded_number q(0e-5,3,1) // 1
  init_bounded_number beta(0.5,0.9,2) // 1
  init_bounded_number r(0e-5,0.4,1) // 2
  init_number log_binit(2) // 2
  init_bounded_number cv_C(0.01,0.8,2)

  number binit
  number NegLogPrior
  vector pred_catch(1,nobs)
  vector biomass(1,nobs+pryears)
  vector f(1,nobs)
  vector fj(1,nobs)
  sdreport_number k
  sdreport_number B_projected
  sdreport_number BmsyB0

  number Bpen;

  objective_function_value ff;

PRELIMINARY_CALCS_SECTION
  // obtengo columnas de datos
  obs_catch = column(data,2);
  cpue = column(data,3);
  years = column(data,1);
  // division entre vectores
  effort = elem_div(obs_catch,cpue);
  // normalizacion / promedio
  avg_effort_p = mean(effort);
  double avg_effort = avg_effort_p;
  effort /= avg_effort;

  //cout << " beta :" << " " <<  beta << endl;

RUNTIME_SECTION
  maximum_function_evaluations 2000, 3000, 4000
  convergence_criteria .00001, 1e-7, 1e-9

PROCEDURE_SECTION
  // Calculo mortalidad por pesca
  calculate_fishing_mortality();
  // Calculo biomasa y captura ajustada
  calculate_biomass_and_predicted_catch();
  // Funcion objetivo
  calculate_the_objective_function();
  // Realizo proyecciones
  if(last_phase())
   {
   Future_projection();
   }


FUNCTION calculate_fishing_mortality
  f = q*effort;

FUNCTION calculate_biomass_and_predicted_catch
  if (!active(log_binit))
  {
    log_binit = log(obs_catch(1)/(q*effort(1)));
  }

  binit = mfexp(log_binit);
  biomass[1] = binit;
  k = binit/beta;

  Bpen = 0;

  for (int i=1; i<=nobs; i++)
  {
    pred_catch(i) = f(i)*biomass(i);
    biomass(i+1) = biomass(i)+(r/((1/m)-1))*(pow(biomass(i),m)/pow(binit,m-1)-biomass(i))- pred_catch(i);
    biomass(i+1) = posfun(biomass(i+1),0.001, Bpen);
  }

  if (last_phase())
     {
     B_projected = biomass(nobs+1);
     BmsyB0 = 1.0/pow(m, 1.0/(m-1.0));
     }


FUNCTION Future_projection
  //Rdev.fill_randn_ni(rec_seed);
  for(int year = nobs + 1; year <= nobs + pryears; year++)
  {
    biomass(year+1) = biomass(year)+(r/((1/m)-1))*(pow(biomass(year),m)/pow(binit,m-1)-biomass(year))- prcatch;
    biomass(year+1) = posfun(biomass(year+1),0.001,Bpen);
  }

FUNCTION calculate_the_objective_function
  double pi=3.14159265358979;

  for(int i=1; i<=nobs ;i++)
    {
    ff += square(log(obs_catch(i))-log(pred_catch(i)))/(2*square(cv_C)) +  log(cv_C) +  0.5*log(2*pi);
    }

  if (initial_params::current_phase<2)
    {
    ff += 1000.*square(log(mean(f)));//  '/.4'
    }

  if (active(beta))
     {
     NegLogPrior = log(beta)+0.5*(log(beta)+0.3566749)*(log(beta)+0.3566749)/(0.3566749*0.3566749);
     }

  ff = ff + NegLogPrior;

  for(int j = 1; j <= nobs; j++)
    {
    fj(j) = pred_catch(j)/biomass(j);
    }

  if(mceval_phase())
    {
    ofstream out("pella_base.mcmc.out",ios::app);
    out << k << " " << r << " " << q << " " << binit << " " << beta << " " << ff << " " << NegLogPrior << " " << B_projected << endl;
    out.close();
    }


REPORT_SECTION
   report << "$Estimated_biomass" << endl;
   report << biomass << endl;
   report << "$Estimated_catch" << endl;
   report << pred_catch << endl;
   report << "$Observed_catch" << endl;
   report << obs_catch << endl;
   report << "$Observed_cpue" << endl;
   report << cpue << endl;
   report << "$Effort" << endl;
   report << effort << endl;
   report << "$Avg_Effort_p" << endl;
   report << avg_effort_p << endl;
   report << "$Avg_Effort" << endl;
   report << avg_effort << endl;
   report << "$Estimated_k" << endl;
   report << k << endl;
   report << "$Fishing_mortality" << endl;
   report << fj << endl;
   report << "$Fishing_mortality_2" << endl;
   report << f << endl;
   report << "$m_beta_r_q_cv_logL_BmsyBo" << endl;
   report << m << " " << beta << " " << r << " " << q << " " << cv_C << " " << ff << " " << BmsyB0 << endl;
   report << "$years_projection" << endl;
   report << pryears << endl;
   report << "$years" << endl;
   report << years << endl;
   report << "$mcnames" << endl;
   report << "k r q Bo Beta  logL logPrior Bact" << endl;
   report << "$mcest" << endl;
   report << k << " " << r << " " << q << " " << binit << " " << beta << " " << ff << " " << NegLogPrior << " " << B_projected << endl;

FINAL_SECTION
   time(&finish);
   elapsed_time = difftime(finish,start);
   hour = long(elapsed_time)/3600;
   minute = long(elapsed_time)/60;
   second = long(elapsed_time);
   cout << endl << "Start time: " << ctime(&start) << endl;
   cout << endl << "finishing time: " << ctime(&finish) << endl;
   cout<<"This run took: "<< endl;
   cout<<hour<<" hours, "<<minute<<" minutes, "<<second<<" seconds."<<endl<<endl<<endl;

